/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <json-glib/json-glib.h>
#include <app_debug.h>
#include <Eina.h>
#include <stdbool.h>
#include <Evas.h>
#include <viewmgr.h>
#include <notification.h>
#include <notification_internal.h>
#include <pwd.h>
#include <gum/gum-user.h>

#include "data_home.h"
#include "datamgr.h"
#include "utils.h"
#include "view.h"
#include "defs.h"

#define STR_SELECT_ACTION_PUSH "push"
#define STR_SELECT_ACTION_LAUNCH "launch"

#define OBJECT_MEMBERS "item"
#define MEMBER_STR_NAME "name"
#define MEMBER_STR_ICON "icon"
#define MEMBER_STR_FOCUS_ICON "focus_icon"
#define MEMBER_STR_SELECT_ACTION "select_action"
#define MEMBER_STR_PARAMETER "parameter"
#define MEMBER_INT_NOTIFICATION "notification"

#define MAX_BUF 64

static inline char *_read_string(JsonReader *reader, char *member)
{
	char *val;

	if (!reader || !member) {
		_ERR("Invalid argument");
		return NULL;
	}

	json_reader_read_member(reader, member);
	val = (char *)json_reader_get_string_value(reader);
	json_reader_end_member(reader);

	return val;
}

static inline gboolean _read_boolean(JsonReader *reader, char *member)
{
	gboolean val;

	if (!reader || !member) {
		_ERR("Invalid argument");
		return false;
	}

	json_reader_read_member(reader, member);
	val = json_reader_get_boolean_value(reader);
	json_reader_end_member(reader);

	return val;
}

static inline void _get_notification_count(notification_type_e type, int *count)
{
	notification_list_h list = NULL;

	notification_get_list(type, -1, &list);
	if (!list)
		return;

	while (list) {
		(*count)++;
		list = notification_list_get_next(list);
	}

	notification_free_list(list);
}

static void _get_notification(char **noti)
{
	int count = 0;
	char buf[MAX_BUF];

	_get_notification_count(NOTIFICATION_TYPE_ONGOING, &count);
	_get_notification_count(NOTIFICATION_TYPE_NOTI, &count);

	if (count > 99)
		count = 99;

	snprintf(buf, sizeof(buf), "%d", count);
	*noti = strdup(buf);
}

static void _get_login_user(char **name, char **icon, char **focus_icon)
{
	GumUser *user;
	struct passwd *passwd;
	uid_t uid = getuid();

	user = gum_user_get_sync(uid, FALSE);
	if (!user) {
		_ERR("failed to get user service");
		passwd = getpwuid(uid);
		*name = passwd->pw_name;
		return;
	}

	g_object_get(G_OBJECT(user), GUM_ATTR_NAME, name, GUM_ATTR_ICON, icon,
			NULL);

	if (!*icon || strlen(*icon) == 0)
		*icon = IMAGE_USER_DEFAULT;

	*focus_icon = (char *)utils_get_focus_icon_from_icon(*icon);

	g_object_unref(user);
}

static struct datamgr_item *_pack_home_item(JsonReader *reader, int i)
{
	struct datamgr_item *di;
	char *name, *icon, *parameter, *focus_icon, *action;
	gboolean noti;

	if (!reader) {
		_ERR("Invalid argument");
		return NULL;
	}

	if (!json_reader_read_element(reader, i)) {
		_ERR("failed to reader %dth element", i);
		return NULL;
	}

	name = _read_string(reader, MEMBER_STR_NAME);
	if (!name)
		goto err;

	icon = _read_string(reader, MEMBER_STR_ICON);
	if (!icon)
		goto err;

	focus_icon = _read_string(reader, MEMBER_STR_FOCUS_ICON);
	if (!focus_icon)
		goto err;

	action = _read_string(reader, MEMBER_STR_SELECT_ACTION);
	parameter = _read_string(reader, MEMBER_STR_PARAMETER);
	noti = _read_boolean(reader, MEMBER_INT_NOTIFICATION);

	json_reader_end_element(reader);

	di = calloc(1, sizeof(*di));
	if (!di) {
		_ERR("failed to calloc home item");
		return NULL;
	}

	if (!strcmp(action, STR_SELECT_ACTION_PUSH)) {
		di->action = ITEM_SELECT_ACTION_PUSH;
		_get_login_user(&name, &icon, &focus_icon);
	} else if (!strcmp(action, STR_SELECT_ACTION_LAUNCH))
		di->action = ITEM_SELECT_ACTION_LAUNCH;
	else
		di->action = ITEM_SELECT_ACTION_MAX;

	di->title = strdup(name);
	di->icon = strdup(icon);
	di->focus_icon = strdup(focus_icon);
	di->parameter = strdup(parameter);
	di->noti = noti;
	if (noti)
		_get_notification(&di->subtitle);

	return di;
err:
	json_reader_end_element(reader);
	return NULL;
}

static Eina_List *_load_home_data(JsonReader *reader)
{
	struct datamgr_item *di;
	int cnt, i;
	Eina_List *list;

	if (!reader) {
		_ERR("Invalid argument");
		return NULL;
	}

	json_reader_read_member(reader, OBJECT_MEMBERS);
	if (!json_reader_is_array(reader)) {
		_ERR(OBJECT_MEMBERS" is not array");
		json_reader_end_member(reader);
		return NULL;
	}

	list = NULL;
	cnt = json_reader_count_elements(reader);
	for (i = 0; i < cnt; i++) {
		di = _pack_home_item(reader, i);
		if (!di)
			continue;

		list = eina_list_append(list, di);
	}

	json_reader_end_member(reader);

	return list;
}

static bool _load_home(struct datamgr *dm)
{
	JsonParser *parser;
	JsonReader *reader;

	parser = json_parser_new();
	if (!parser) {
		_ERR("failed to new parser");
		return false;
	}

	if (!json_parser_load_from_file(parser, DEFCONFIG, NULL)) {
		_ERR("failed to load from file");
		g_object_unref(parser);
		return false;
	}

	reader = json_reader_new(json_parser_get_root(parser));
	if (!reader) {
		_ERR("failed to new reader");
		g_object_unref(parser);
		return false;
	}

	dm->list = _load_home_data(reader);
	if (!dm->list) {
		_ERR("failed to load home data");
		g_object_unref(reader);
		g_object_unref(parser);
		return false;
	}

	g_object_unref(reader);
	g_object_unref(parser);

	return true;
}

static void _unload_home(struct datamgr *dm)
{
	struct datamgr_item *di;

	EINA_LIST_FREE(dm->list, di) {
		free(di->title);
		free(di->icon);
		free(di->focus_icon);
		free(di->parameter);
		free(di->subtitle);

		free(di);
	}

	dm->list = NULL;
}

static Eina_List *_get_items(struct datamgr *dm)
{
	if (!dm) {
		_ERR("Invalid argument");
		return NULL;
	}

	_unload_home(dm);
	_load_home(dm);

	return dm->list;
}

static void _notification_cb(void *data, notification_type_e type)
{
	char *noti = NULL;
	Eina_List *l;
	struct datamgr *dm;
	struct datamgr_item *di;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	dm = data;

	_get_notification(&noti);
	viewmgr_update_view(VIEW_HOME, UPDATE_TYPE_NOTI, noti);

	EINA_LIST_FOREACH(dm->list, l, di) {
		if (di->noti)
			di->subtitle = noti;
	}
}

static void _fini(struct datamgr *dm)
{
	if (!dm) {
		_ERR("Invalid argument");
		return;
	}

	_unload_home(dm);

	notification_unresister_changed_cb(_notification_cb);
}

static bool _init(struct datamgr *dm)
{
	if (!dm) {
		_ERR("Invalid argument");
		return false;
	}

	notification_resister_changed_cb(_notification_cb, dm);

	return _load_home(dm);
}

static void _select(struct datamgr_item *di)
{
	if (!di || !di->parameter)
		return;

	switch (di->action) {
	case ITEM_SELECT_ACTION_LAUNCH:
		utils_launch_app(di->parameter, NULL, NULL);
		break;
	case ITEM_SELECT_ACTION_PUSH:
		viewmgr_push_view(di->parameter);
		break;
	default:
		_ERR("Invalid state");
		return;
	}
}

static struct data_class dclass = {
	.init = _init,
	.fini = _fini,
	.get_items = _get_items,
	.select = _select
};

struct data_class *datamgr_home_get_dclass(void)
{
	return &dclass;
}

